# -*- coding: utf-8 -*-
"""
app.api.items
~~~~~~~~~~~~~

This module implements the items API.

:copyright: (c) 2017 European Spallation Source ERIC
:license: BSD 2-Clause, see LICENSE for more details.

"""
import sqlalchemy as sa
from flask import (current_app, Blueprint, jsonify, request)
from flask_jwt_extended import create_access_token, jwt_required
from flask_ldap3_login import AuthenticationResponseStatus
from ..extensions import ldap_manager, db
from ..models import User, Item
from .. import utils

bp = Blueprint('api', __name__)


@bp.route('/login', methods=['POST'])
def login():
    try:
        username = request.get_json()['username']
        password = request.get_json()['password']
    except (TypeError, KeyError):
        return jsonify({'msg': 'Bad Request'}), 400
    response = ldap_manager.authenticate(username, password)
    if response.status == AuthenticationResponseStatus.success:
        current_app.logger.debug(f'{username} successfully logged in')
        user = User.query.filter_by(username=username).first()
        if user is None:
            # LDAP user not in database yet - create it
            user = User(username, name=response.user_info['cn'], email=response.user_info['mail'])
            db.session.add(user)
            db.session.commit()
        payload = {'access_token': create_access_token(identity=user.id)}
        return jsonify(payload), 200
    return jsonify({'msg': 'Invalid credentials'}), 401


@bp.route('/items')
@jwt_required
def get_items():
    # TODO: add pagination
    items = Item.query.order_by(Item._created)
    data = [item.to_dict() for item in items]
    return jsonify({'items': data})


@bp.route('/items', methods=['POST'])
@jwt_required
def create_item():
    data = request.get_json()
    if data is None:
        raise utils.InventoryError('Body should be a JSON object')
    if 'serial_number' not in data:
        raise utils.InventoryError('serial_number is mandatory')
    try:
        item = Item(**data)
    except TypeError as e:
        message = str(e).replace('__init__() got an ', '')
        raise utils.InventoryError(message)
    db.session.add(item)
    try:
        db.session.commit()
    except sa.exc.IntegrityError as e:
        db.session.rollback()
        raise utils.InventoryError('IntegrityError', status_code=409)
    return jsonify(item.to_dict()), 201
